#!/bin/bash
# detect timestamp for a CVS tag
# compare CVS bydate / CVS by tag / BZR by date

tz=$TZ
export TZ=GMT

pwd=$PWD
trunk=""

export CVSROOT=/home/adrian/work/cvs2bzr/cvsrepo
bzrroot=/home/adrian/work/cvs2bzr/bzrrepo
cvsroot=$CVSROOT

#test arguments
if [ $# -lt 2  ] ; then
   echo "Usage: u1-detecttag.sh module_name tag_name [log_dir]"
   exit 1
else
   module=$1
   tag=$2   
fi

# Test CVS module
if [ ! -d $cvsroot/$module ] ;then
   echo "$cvsroot/$module does not exist.";
   exit 1
fi

# Test BZR module
if [ ! -d $bzrroot/$module ] ;then
   echo "$bzrroot/$module does not exist."
   exit 1
fi
workdir=$pwd/tmp-detecttag
rm -fr $workdir
mkdir $workdir

###############################################################
#                      STAGE 1                                #
#                                                             #
# checkout from CVS by tag and detect last modification time  #
#                                                             #
###############################################################

# Checkout from CVS by tag
echo "CVS checkout by tag $tag"
rm -fr $workdir/cvsbytag
mkdir $workdir/cvsbytag
cd $workdir/cvsbytag

cvs -Q export -r $tag $module
# as the -P seems to be unavailable - delete all empty directories
find $workdir/cvsbytag -type d -empty -exec rmdir {} \;

# Detect last modification time
cd $workdir/cvsbytag/$module
filem=$(find . -type f -printf "%T@\0%p\0" | awk '
    {
        if ($0>max) {
            max=$0; 
            getline mostrecent
        } else 
            getline
    } 
    END{print mostrecent} ' RS='\0')


#last modification time
lmd=$(stat -c %Y $filem)

# GMT BZR date

# The ":date" option in bazaar means:
#  "Matches the first entry after a given date"
# The commit time is later(or equal) to this(lmd) date
# Add 1 sec to avoid the equal case 
blmd=$((lmd+1))
# The bazaar revision could be found with before:date:blmd
# TODO : last revision 

vdatenext=$(date -d "1970-01-01 + $blmd secs" +%F)
vtimenext=$(date -d "1970-01-01 + $blmd secs" +%T)

###############################################################
#                      STAGE 2                                #
#                                                             #
# checkout from BZR by date                                   #
#                                                             #
###############################################################

# Checkout from BZR
echo "BZR checkout by date $bdate"
rm -fr  $workdir/bzrbydate
mkdir  $workdir/bzrbydate
mkdir  $workdir/bzrbydate/$module
cd $workdir/bzrbydate/$module

revno=$(bzr log -r before:date:"$vdatenext,$vtimenext" $bzrroot/$module/$trunk | \
       grep revno | \
       awk '{print $2}')

echo "Bazaar revno=" $revno
bzr checkout --lightweight -q -r $revno $bzrroot/$module/$trunk .

###############################################################
#                      STAGE 3                                #
#                                                             #
# checkout from CVS by date                                   #
#                                                             #
###############################################################
  
# The "-D date" option for CVS means:
#  "Use the most recent revision no later than date"  

echo "time+1:" $vdatenext $vtimenext

# Checkout from CVS by date
echo "CVS checkout by date $dcvs"
rm -fr $workdir/cvsbydate
mkdir $workdir/cvsbydate
cd $workdir/cvsbydate

cvs -Q export -D"$vdatenext $vtimenext GMT" $module
find $workdir/cvsbydate -type d -empty -exec rmdir {} \;

###############################################################
#                      STAGE 4                                #
#                                                             #
# compare versions two by two                                 #
#                                                             #
###############################################################

# Diff BZR date / CVS date
diff -rq -x CVS -x .bzr -I '@version $Id' \
                         $workdir/bzrbydate/$module \
                         $workdir/cvsbydate/$module > \
                         $workdir/out1.diff
# Test if there are diffs
[ -s $workdir/out1.diff ] && diff1="DIFF" || diff1="-OK-"
echo "Diff BZR date / CVS date ... $diff1"

# Diff BZR date / CVS tag
diff -rq -x CVS -x .bzr -I '@version $Id' \
                         $workdir/bzrbydate/$module \
                         $workdir/cvsbytag/$module > \
                         $workdir/out2.diff
grep '^Only in '  $workdir/out2.diff >  $workdir/out2-only.diff
grep -v '^Only in '  $workdir/out2.diff >  $workdir/out2.diff.tmp
mv -f $workdir/out2.diff.tmp $workdir/out2.diff
# Test if there are diffs
[ -s $workdir/out2.diff ] && diff2="DIFF" || diff2="-OK-"
echo "Diff BZR date / CVS tag  ... $diff2"

# Diff CVS date / CVS tag
diff -rq -x CVS -x .bzr $workdir/cvsbydate/$module \
                        $workdir/cvsbytag/$module > \
                        $workdir/out3.diff
# Test if there are diffs
[ -s $workdir/out3.diff ] && diff3="DIFF" || diff3="-OK-"
echo "Diff CVS date / CVS tag  ... $diff3"

if [ ! -z $3 ]; then   
   echo "Write log to $3/log.diff"
   cd $pwd
   mkdir -p $3/$tag   
   cp $workdir/*.diff $3/$tag
   echo $diff1 $diff2 $diff3 $tag >> $3/log.diff 
fi 

